from abc import *

class IDownloader:
    @abstractmethod
    def download(self, line):
        pass

    @property
    @abstractmethod
    def DANMU_TYPE(self):
        pass

    @property
    @abstractmethod
    def DANMU_EXTNAME(self):
        pass

    @property
    @abstractmethod
    def DANMU_LIST_EXTNAME(self):
        pass


import os
import time


__all__ = ['download_line', 'download_file', 'download_dir']


def download_line(line, tool, add_file_timestamp=False):
    line_res = tool.download(line)
    for item in line_res:
        filename = item['filename']
        if add_file_timestamp:
            tmp_1 = os.path.splitext(filename)
            timestamp = time.strftime('%Y%m%d%H%M%S', time.localtime())
            filename = tmp_1[0] + '_' + timestamp + tmp_1[1]
        with open(filename, 'wb') as f:
            f.write(item['data'])


def download_file(filename, tool, add_dir_timestamp=False, add_file_timestamp=False):
    print('Start process list file: "%s".' % filename)
    tmp_1 = os.path.splitext(filename)
    if tmp_1[1] == '.' + tool.DANMU_LIST_EXTNAME:
        out_dir = tmp_1[0]
    else:
        out_dir = filename
    if add_dir_timestamp:
        timestamp = time.strftime('%Y%m%d%H%M%S', time.localtime())
        out_dir = out_dir + '_' + timestamp
    if not os.path.exists(out_dir):
        os.mkdir(out_dir)
    curdir = os.path.abspath(os.curdir)
    try:
        with open(filename, encoding='utf8') as f:
            os.chdir(out_dir)
            for line in f:
                download_line(line, tool, add_file_timestamp=add_file_timestamp)
    except Exception as e:
        print(e)
        return
    os.chdir(curdir)
    print('Success generate dir: "%s" for list file.' % out_dir)


def download_dir(dirname, tool, add_dir_timestamp=False, add_file_timestamp=False):
    print('Start process dir: "%s".' % dirname)
    for filename in os.listdir(dirname):
        filename = os.path.join(dirname, filename)
        if not os.path.isfile(filename):
            continue
        if os.path.splitext(filename)[1] != '.' + tool.DANMU_LIST_EXTNAME:
            continue
        download_file(filename, tool, add_dir_timestamp=add_dir_timestamp, add_file_timestamp=add_file_timestamp)
    print('Success process dir: "%s".' % dirname)


import urllib.request
import zlib

from danmu_utils.common.IDownloader import IDownloader


class BilibiliDownloader(IDownloader):
    @property
    def DANMU_TYPE(self):
        return 'bilibili'

    @property
    def DANMU_EXTNAME(self):
        return 'xml'

    @property
    def DANMU_LIST_EXTNAME(self):
        return 'xmllist'

    def __deflate(self, data):
        try:
            return zlib.decompress(data, -zlib.MAX_WBITS)
        except zlib.error:
            return zlib.decompress(data)

    def _download(self, aid=None):
        if aid != None:
            url = 'https://comment.bilibili.com/%s.xml' % (aid)
            try:
                with urllib.request.urlopen(url) as f:
                    danmu = f.read()
                try:
                    danmu_deflate = self.__deflate(danmu)
                    return danmu_deflate
                except Exception as e:
                    print(e)
            except Exception as e:
                print(e)

    def download(self, line):
        line_res = []
        line_params = line.strip('\n').split('\t')
        print('Start process item: "%s".' % line.strip('\n'))
        if len(line_params) < 1:
            return False
        if not line_params[0].isdecimal():
            return False
        aid = line_params[0]
        res = self._download(aid=aid)
        if res != None:
            item_res = {}
            item_res['filename'] = aid + '.' + self.DANMU_EXTNAME
            item_res['data'] = res
            line_res.append(item_res)
        return line_res


from danmu_utils.common.plugin_collection import add_download_tool
add_download_tool(BilibiliDownloader().DANMU_TYPE, BilibiliDownloader)


__all__ = ['add_download_tool', 'add_convert_tool', 'get_download_tool', 'get_convert_tool', 'print_download_tools', 'print_convert_tools']

download_tools = {}
convert_tools = {}

def add_download_tool(name, obj):
    download_tools[name] = obj

def add_convert_tool(src_name, dst_name, obj):
    name = "%s->%s" % (src_name, dst_name)
    convert_tools[name] = obj

def get_download_tool(name):
    return download_tools[name]

def get_convert_tool(src_name, dst_name):
    name = "%s->%s" %(src_name, dst_name)
    return convert_tools[name]

def print_download_tools():
    print(download_tools)

def print_convert_tools():
    print(convert_tools)


from abc import *


class IConverter:
    @abstractmethod
    def converter(self, data):
        pass

    @property
    @abstractmethod
    def DANMU_TYPE_SRC(self):
        pass

    @property
    @abstractmethod
    def DANMU_TYPE_DST(self):
        pass

    @property
    @abstractmethod
    def DANMU_EXTNAME_SRC(self):
        pass

    @property
    @abstractmethod
    def DANMU_EXTNAME_DST(self):
        pass


import json
from xml.dom import minidom
from danmu_utils.common.IConverter import IConverter


class DiyidanToBilibiliConverter(IConverter):
    @property
    def DANMU_TYPE_SRC(self):
        return 'diyidan'

    @property
    def DANMU_TYPE_DST(self):
        return 'bilibili'

    @property
    def DANMU_EXTNAME_SRC(self):
        return 'dydjson'

    @property
    def DANMU_EXTNAME_DST(self):
        return 'xml'

    def _convert_entry(self, entry):
        try:
            if 'text' in entry:
                dst_text = entry['text']
                dst_time = entry['time'] / 10
                dst_color = int(entry['color'].split('#')[1], 16)
                dst_sender = entry['danmakuId']
            elif 'danmakuContent' in entry:
                dst_text = entry['danmakuContent']
                dst_time = entry['danmakuTime'] / 1000
                dst_color = entry['danmakuTextColor']
                dst_sender = entry['danmakuId']
        except Exception as e:
            print(e)
            return None
        xml = minidom.Document()
        dst_entry = xml.createElement('d')
        p = '%f,%d,%d,%d,%d,%d,%d,%d' % (
            dst_time,  # 1. 弹幕发送相对视频的时间（以前是以秒为单位的整数，现在用浮点记了，更精准）
            1,  # 2. 弹幕类型：1~3（但貌似只见过1）滚动弹幕、4底端弹幕、5顶端弹幕、6逆向弹幕、7精准定位、8高级弹幕【默认是1，基本以1、4、5多见】
            25,  # 3. 字号：12非常小,16特小,18小,25中,36大,45很大,64特别大【默认是25】
            dst_color,  # 4. 字体颜色：不是RGB而是十进制
            0,  # 5. 弹幕发送时的UNIX时间戳，基准时间1970-1-1 08:00:00
            0,  # 6. 弹幕池类型：0普通 1字幕 2特殊
            dst_sender,  # 7. 发送者ID【注意不是uid，具体怎么关联的还不清楚，屏蔽用】
            0  # 8. 弹幕在数据库的ID
        )
        dst_entry.setAttribute('p', p)
        dst_entry.appendChild(xml.createTextNode(dst_text))
        return dst_entry

    def convert(self, data):
        try:
            item_src = json.loads(data)
        except Exception as e:
            print(e)
            return None
        xml = minidom.Document()
        list = xml.createElement('i')
        xml.appendChild(list)
        for entry_src in item_src['data']['danmakuList']:
            entry_dst = self._convert_entry(entry_src)
            if (entry_dst == None):
                continue
            list.appendChild(entry_dst)
        item_dst = xml.toprettyxml(encoding='UTF-8')
        return item_dst


from danmu_utils.common.plugin_collection import add_convert_tool

add_convert_tool(DiyidanToBilibiliConverter().DANMU_TYPE_SRC, DiyidanToBilibiliConverter().DANMU_TYPE_DST, DiyidanToBilibiliConverter)



import urllib.request

from danmu_utils.common.IDownloader import IDownloader


class DiyidanDownloader(IDownloader):
    @property
    def DANMU_TYPE(self):
        return 'diyidan'

    @property
    def DANMU_EXTNAME(self):
        return 'dydjson'

    @property
    def DANMU_LIST_EXTNAME(self):
        return 'dydlist'

    def _download(self, videoId=None, postId=None):
        res = {}
        if videoId != None:
            url = 'https://api.diyidan.net/v0.2/posts/danmaku?videoId=%s' % (videoId)
            try:
                with urllib.request.urlopen(url) as f:
                    danmu = f.read()
                    res['videoId'] = danmu
            except Exception as e:
                print(e)
        if postId != None:
            url = 'https://api.diyidan.net/v0.2/posts/danmaku?postId=%s' % (postId)
            try:
                with urllib.request.urlopen(url) as f:
                    danmu = f.read()
                    res['postId'] = danmu
            except Exception as e:
                print(e)
        return res

    def download(self, line):
        line_res = []
        line_params = line.strip('\n').split('\t')
        print('Start process line: "%s".' % line.strip('\n'))
        if len(line_params) < 2:
            return False
        if not line_params[0].isdecimal():
            return False
        if not line_params[1].isdecimal():
            return False
        videoId = line_params[0]
        postId = line_params[1]
        res = self._download(videoId=videoId, postId=postId)
        if 'videoId' in res:
            out_filename = ''
            if postId != None:
                out_filename = out_filename + postId
            out_filename = out_filename + '-' + videoId
            out_filename = out_filename + '.' + 'dydjson'
            item_res = {}
            item_res['filename'] = out_filename
            item_res['data'] = res['videoId']
            line_res.append(item_res)
        if 'postId' in res:
            out_filename = postId + '-'
            out_filename = out_filename + '.' + 'dydjson'
            item_res = {}
            item_res['filename'] = out_filename
            item_res['data'] = res['postId']
            line_res.append(item_res)
        return line_res

if __name__ == '__main__':
    pass
else:
    from danmu_utils.common.plugin_collection import add_download_tool
    add_download_tool(DiyidanDownloader().DANMU_TYPE, DiyidanDownloader)


import os


__all__ = ['convert_file', 'convert_dir']


def convert_file(filename, tool):
    print('Start process file: "%s".' % filename)
    try:
        with open(filename, encoding='utf8') as f:
            danmu_diyidan = f.read()
    except Exception as e:
        print(e)
        return False
    # Determine output file name
    tmp_1 = os.path.splitext(filename)
    if tmp_1[1] == '.' + tool.DANMU_EXTNAME_SRC:
        out_filename = tmp_1[0] + '.' + tool.DANMU_EXTNAME_DST
    else:
        out_filename = filename + '.' + tool.DANMU_EXTNAME_DST
    try:
        danmu_out = tool.convert(danmu_diyidan)
        with open(out_filename, mode='wb') as f:
            f.write(danmu_out)
    except Exception as e:
        print(e)
        return False
    print('Success generate file: "%s".' % out_filename)
    return True


def convert_dir(dirname, tool):
    for filename in os.listdir(dirname):
        filename = os.path.join(dirname, filename)
        if not os.path.isfile(filename):
            continue
        if os.path.splitext(filename)[1] != '.' + tool.DANMU_EXTNAME_SRC:
            continue
        convert_file(filename, tool)


import urllib.request
import urllib
from urllib.parse import urlencode
import json
import re

from danmu_utils.common.IDownloader import IDownloader


class YoukuDownloader(IDownloader):
    @property
    def DANMU_TYPE(self):
        return 'youku'

    @property
    def DANMU_EXTNAME(self):
        return 'ykjson'

    @property
    def DANMU_LIST_EXTNAME(self):
        return 'yklist'

    def _download(self, videoId):
        param_now = {
            "mcount": "1",
            "ct": "1001",
            "iid": "",
        }
        danmu_collect = {
            "count": 0,
            "filtered": 1,
            "result": [],
        }
        param_now["iid"] = videoId
        j = 0
        while(True):
            param_now["mat"] = j
            query = urlencode(param_now)
            url = "https://service.danmu.youku.com/list?%s" % query
            print('Start download: "%s".' % url)
            try:
                with urllib.request.urlopen(url) as f:
                    body = f.read()
            except Exception as e:
                print(e)
                break
            danmus = re.findall(r"\{.*\}", str(body, encoding='utf-8'))
            danmu = danmus[0]
            try:
                danmu_json = json.loads(danmu)
                if danmu_json["count"] == 0:
                    print("Section download finished: %s" % videoId)
                    break
            except Exception as e:
                print(e)
                break
            try:
                danmu_collect["count"] += danmu_json["count"]
                danmu_collect["result"].extend(danmu_json["result"])
            except Exception as e:
                print(e)
                break
            j += 1
        return json.dumps(danmu_collect, ensure_ascii=False).encode(encoding='utf-8')

    def download(self, line):
        line_res = []
        line_params = line.strip('\n').split('\t')
        print('Start process item: "%s".' % line.strip('\n'))
        if len(line_params) < 1:
            return line_res
        if not line_params[0].isdecimal():
            return line_res
        videoId = line_params[0]
        res = self._download(videoId)
        if res != None:
            item_res = {}
            item_res['filename'] = videoId + '.' + self.DANMU_EXTNAME
            item_res['data'] = res
            line_res.append(item_res)
        return line_res


from danmu_utils.common.plugin_collection import add_download_tool
add_download_tool(YoukuDownloader().DANMU_TYPE, YoukuDownloader)